;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x429cdbd537844fc7bf1ac52b3445dfcb . "./code.cg2js/cg-status-bar.fasl")
(#x6ef2e1b33f100400b6fdf491987dffa0 . "./code.cg2js/cg-hotspot.fasl")
(#x5bf66301ebac66ec4fba971dc20ce71c . "./code.cg2js/cg-toolbar.fasl")
(#x0d4ad8906a9a073e212375f25d10317b . "./code.cg2js/cg-clipboard-stack.fasl")
(#x739c2c8be1b41538ce4a9920d94e1a1c . "./code.cg2js/cg-tooltip.fasl")
(#x8854dc546903dc309ae247a7be9d242c . "./code.cg2js/cg-calendar.fasl")
(#x8ef9c9fa62471c14c47b3584134d2b34 . "./code.cg2js/cg-item-list.fasl")
(#xd3f7bb9e965673f7d2d2ab1cfe92d5dd . "./code.cg2js/cg-bitmap-stream.fasl")
(#x7fbc24241491dcbf71b93966204a936b . "./code.cg2js/cg-trackbar.fasl")
(#xebaa010ade62ea58f2bc937cf4ea7731 . "./code.cg2js/cg-class-grid.fasl")
(#x6818c8965dfc0bbd61c989856affbcd7 . "./code.cg2js/cg-dialog-item.fasl")
(#xed92da6f111243b2afc8d1feba7410f9 . "./code.cg2js/cg-comtab.fasl")
(#x97728192372866a7d177fdc843689df5 . "./code.cg2js/cg-lisp-text.fasl")
(#x07bc2d8350256c5917832892513c0af4 . "./code.cg2js/cg-button.fasl")
(#xb3634f7911f2e41fbf07ccad627a88d7 . "./code.cg2js/cg-printing.fasl")
(#x9d17f28e568e5b32157e8d3bc3b6b699 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xe247bfe10655a895b185a40952466cfc . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x8340cd191edf4fff24c131f711bdf001 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xb94bc4decf380188959a0dcd3136a1e6 . "./code.cg2js/cg-selected-object.fasl")
(#x7e8f7da71ada633e11931dfa5e614536 . "./code.cg2js/cg-text-or-combo.fasl")
(#x262cfac5776a9f3c47000369c3eba0c6 . "./code.cg2js/cg-graphics-context.fasl")
(#x7a9f0dda91d5a6c3a7b5db181325e40d . "./code.cg2js/cg-edit-in-place.fasl")
(#x7d701ce16cf67e819164f283ff8be6e7 . "./code.cg2js/cg-string-dialog.fasl")
(#x1842433d0357eb2411e551f828f3ff82 . "./code.cg2js/cg-list-view.fasl")
(#xb9c5b5807db9e9fbf96e394b527c2201 . "./code.cg2js/cg-intersect.fasl")
(#x92352e10f078d51f9cbb079068b37337 . "./code.cg2js/cg-text-widget.fasl")
(#x574541986e56d230ea25f95099ff319a . "./code.cg2js/cg-directory-dialog.fasl")
(#x321eaaf6d331b4d23e08177de966f7ee . "./code.cg2js/cg-file-dialog.fasl")
(#xad8a91af54bfb86bc8da2ad6fdd07ecf . "./code.cg2js/cg-dde-utils.fasl")
(#x0eee95bb327f1c681253b78409a222db . "./code.cg2js/cg-nodes.fasl")
(#xec1e2dc0cba0b3ee155a415daba93f9b . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x20d5eeafa17274884d91117ffee6e8a1 . "./code.cg2js/cg-up-down-control.fasl")
(#xd0dcedbc437a9fa56a32d3ceaf2117f2 . "./code.cg2js/cg-get-position.fasl")
(#xfad7346802713b66cae03d8fda3490a3 . "./code.cg2js/cg-toggling-widget.fasl")
(#xd73bf5128ac148809ad85be56b1eb01f . "./code.cg2js/cg-acache.fasl")
(#x68782d62180a1ecbfaa7b3aed20328f4 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x8faa470f3d6ed1404a713db8decc6af0 . "./code.cg2js/cg-base.fasl")
(#x2980e35d56e8da48fc7a269e0e046d00 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x3588eb66ba6d6689db5af3b4b224e305 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x24304fce209d17e03c9fadc75bade12d . "./code.cg2js/cg-property.fasl")
(#x99e82eae76d50fd8c69be952f84dded3 . "./code.cg2js/cg-editable-text.fasl")
(#x734f654d060dcfc7dcb0703d053dbc2d . "./code.cg2js/cg-lamp.fasl")
(#xa86addee5a0deb2d9b29128c641ae554 . "./code.cg2js/cg-progress-indicator.fasl")
(#xd056becfd1d17b639898005c2641c3ee . "./code.cg2js/cg-chart-widget.fasl")
(#xd406ccd51a513161527c273df2e8711b . "./code.cg2js/cg-common-control.fasl")
(#xe34a3b22caf766c1db06a5f82c9a3850 . "./code.cg2js/cg-font-dialog.fasl")
(#xe7a178d2d3b1d43f030802087a2b21a5 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x99feebdd400e0a03dd7e63ccf6a51d84 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xae0688a76d6fe32597b2e1e88279f5e2 . "./code.cg2js/cg-os-window.fasl")
(#x11cc5de182b1c6b606c3f25ce2652973 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x7dda589de346d632e4775342e2d2ccf0 . "./code.cg2js/cg-pixmap.fasl")
(#xcaae4687d6250b4701093cc9a2f91c45 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x674b277d04108c2e6a582c6c0d9d9488 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xb50419812e8caafe112beabf4fb800d2 . "./code.cg2js/cg-scaling-stream.fasl")
(#x092809f8274ba0452e134972bb9220ce . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xe18a618ee0d88dc75fa5fd49337b691a . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xc3e023a45927c69e8039d329d3691013 . "./code.cg2js/cg-timer.fasl")
(#x1ddc5a4bf5a4eb6c706e38c4b877d447 . "./code.cg2js/cg-scroll-bar.fasl")
(#x68d9784f98e7222c1df7a80abd37e8eb . "./code.cg2js/cg-caret.fasl")
(#x570f5564cedf12529327a0550988c60d . "./code.cg2js/cg-template-string.fasl")
(#x739a471af47f2f9010d45746169e7567 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x403fe693e7454273457e88e660161175 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xf67ce74fc4ee8bfe722e2e96b13b659f . "./code.cg2js/cg-combo-box.fasl")
(#xd1b93a30a70bea6cdc69c452f1ba0d47 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xdc3712a700833075d9978459284a7a53 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xa1560c7f94d088f38bb0d61ba397006b . "./code.cg2js/cg-menu-tooltip.fasl")
(#x7f2fe52e8655e5f16e492f42f94d0bef . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x40f244406724c7d750d36cb2652b2545 . "./code.cg2js/cg-player.fasl")
(#x87c2c64aab7fb1d97775933b73fad62f . "./code.cg2js/cg-outline.fasl")
(#x33d122e2fb2d4bb9ab8c9261e048446a . "./code.cg2js/cg-class-support.fasl")
(#x61397de06f10739156014392e49d1415 . "./code.cg2js/cg-lettered-menu.fasl")
(#xfcd0a9eb10c885eb4e7653482e870b50 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x11b3f8a0b3524bbe977413391a8e89b5 . "./code.cg2js/cg-fill-texture.fasl")
(#x482dd5d1aaa6e878842f4587df01d478 . "./code.cg2js/cg-lisp-widget.fasl")
(#xe56ff905224a79a980b9b8ec9c033efb . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xc50e94acd6a40327dc8c1f9ec0a44f1e . "./code.cg2js/cg-clipboard.fasl")
(#x1af42273c94da3b0c723d23e7323554a . "./code.cg2js/cg-wrap-string.fasl")
(#x4a7420480256f3484857e641a688b4f4 . "./code.cg2js/cg-os-widget.fasl")
(#x3a7ea72dfdcc0066955524dd9cb2a21f . "./code.cg2js/cg-check-box.fasl")
(#x41ec455607c6b5e6a44de133a0cf26fe . "./code.cg2js/cg-choice-list.fasl")
(#xc8e63ea54b4f123d0e9b31bf5f289c97 . "./code.cg2js/cg-layout.fasl")
(#xdb1bab5d051b14ab7337a6e0f2d3cc96 . "./code.cg2js/cg-group-box.fasl")
(#xec38aa36c132b1ea757d415a97cc48cc . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x79423705fe07a2b404af858787395689 . "./code.cg2js/cg-radio-button.fasl")
(#x83eebdd6fe3faad31b7d0fcfcae214d8 . "./code.cg2js/cg-tab-control.fasl")
(#x226c5aebdc4716ab044300297ddfc6d6 . "./code.cg2js/cg.fasl")
(#xc99a0e9fec6e382849ded486d01c583f . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xf8412cff6f631afbbb80f617adf99901 . "./code.cg2js/cg-dropping-outline.fasl")
(#x818c8174b3f02e434663d2a3f86e7742 . "./code.cg2js/cg-get-pixmap.fasl")
(#xc5820feb9a54f9480ab0468bf9654dd0 . "./code.cg2js/cg-grid-widget.fasl")
(#x33d74c606e2c9d66cc7016cc42fe73af . "./code.cg2js/cg-utility-dialog.fasl")
(#x4f264d5ad320fd2107ec37d9625c8592 . "./code.cg2js/cg-static-text.fasl")
(#xe5a8ebbdce0177162cc75443c1212272 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xa2d0c991e7b272cece646a86bbf85056 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x6ba2f27aea1a182aed9388f2959c9551 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x413b3e4e88cf84eaec10a459af6e7551 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xe42d86a2153bffd643c88ea1fce766e6 . "./code.cg2js/cg-plot-widget.fasl")
(#xcb55ca8623bc676a2c15ae4c9af10202 . "./code.cg2js/cg-color-dialog.fasl")
(#x6f74d40d98b9deb994553bf98054f1e1 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xed4b018e37063d70b7cf44399ddc5917 . "./code.cg2js/cg-icon.fasl")
(#xf517fbb13170b689e171ae10cefaa495 . "./code.cg2js/cg-dev.fasl")
(#x449decbd465e09f18118315fe9d05c35 . "./code.cg2js/cg-choose-printer.fasl")
(#x8c84f1afe50c5ea4bccf434701cbcc45 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x4a726e0a8da619b02b509bc9d67c975b . "./code.cg2js/cg-menu.fasl")
(#x9966c8a028b1d39131d61c6cbc038fd7 . "./code.cg2js/cg-yes-no-list.fasl")
(#xf2f318cc8de8561f9a88451a03f6885b . "./code.cg2js/cg-palette.fasl")
(#x8581b0a7e8c05d653700a3068cd4e367 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xd321419708a304b5e2b27b0b6cff012a . "./code.cg2js/cg-message-dialog.fasl")
(#xa20aa6c6791591a6d38a38234708d00b . "./code.cg2js/cg-shortcut-menu.fasl")
(#x33b9947f9ae3f818177b5e36f3911438 . "./code.cg2js/cg-split-bar.fasl")
(#x29d68ca1f0870ea464d99ae59f85ea2a . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x641b95cd9d96bc886b211c3942f77a16 . "./code.cg2js/cg-web-browser.fasl")
(#xcfaf468589ffde1820e8ad06e4597ec7 . "./code.cg2js/cg-project-window.fasl")
(#x3d563a2eb2c0f147b571c67aedc5e101 . "./code.cg2js/cg-picture-widget.fasl")
(#xb6aabb55a72ca5c2cd531c3d2f07373b . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x28828e272afc4a2d4e3c61edfc367b38 . "./code.cg2js/cg-object-editor.fasl")
(#x844afc8ef5a262e59454635d2c0c56fe . "./code.cg2js/cg-paren-matching.fasl")
(#x5cd9e4c54d8f5e2d6f8e0f8a581a6c13 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xe3905a4627c2bace8ff3a9182d958bfe . "./code.cg2js/cg-drawable.fasl")
(#x7caf3ebf08f645723038ee97e194679f . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x7da5bc75a743bae8334773a3aa9ddcf5 . "./code.cg2js/cg-html-widget.fasl")
(#x2a47c925048af8147782cda19920b72f . "./code.cg2js/cg-curve.fasl")
(#xc090283028589e031307f425395a88ba . "./code.cg2js/cg-yes-no-string.fasl")
(#xcb54961d1ef14c67eba136904510a717 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xd0e0ba536d68ffd0527f21575d346735 . "./code.cg2js/cg-html-dialog.fasl")
(#x187d3a41d5167c3ee41bb427e6be2baf . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xe58fa438939ad6731a662e98d54a88f9 . "./code.cg2js/cg-tray.fasl")
(#x31e00415ca1d47ec90aa10a0534979c8 . "./code.idejs/ide-apropos.fasl")
(#x12b2d7e998248bca2267a423cf9ac26d . "./code.idejs/ide-window-list.fasl")
(#x6f8480b5e3046f4e84d305176cd21371 . "./code.idejs/ide-class-browser.fasl")
(#xf5d7fc3cfb98e2dff18a258c1ea7c536 . "./code.idejs/ide-base.fasl")
(#x3d2588dfb3d4c3294208c2e459a0f02e . "./code.idejs/ide-process-browser.fasl")
(#xf51451d74dd7c20924289c272d1d1761 . "./code.idejs/ide-acache.fasl")
(#x49ea92ef777026eb418d305baa0bfc80 . "./code.idejs/ide-navigator.fasl")
(#x749275df0de7b85da8ab439832c5717b . "./code.idejs/ide-trace.fasl")
(#xc0d10fac6b3769ba897f1f53a733d49b . "./code.idejs/ide-project.fasl")
(#x204a3371656902c5e2b794532b32f36a . "./code.idejs/ide-patch-dialog.fasl")
(#xfd02c01748b23288d96934f6ada296b6 . "./code.idejs/ide-grapher.fasl")
(#xa26b481f386185927f036c89e479032d . "./code.idejs/ide-dev.fasl")
(#xcb74c930d3462cd410e70ec2b6a6dbdd . "./code.idejs/ide-keyshort.fasl")
(#x849b0f242670f2b2a968eb56835693d2 . "./code.idejs/ide-editor.fasl")
(#x09f5915999aa2787b1cef34833baaa2c . "./code.idejs/ide-inspector.fasl")
(#x770862d839d7a0b21d1f7848108c6f67 . "./code.idejs/ide.fasl")
(#xe09905c6ff35034c8917524f19e6467b . "./code.idejs/ide-clipboard.fasl")
(#x13c3b95f10d807b35cd11fb5c1baf384 . "./code.idejs/ide-find-definitions.fasl")
(#x160bc3d88e867b3165abfe0c040382fc . "./code.idejs/ide-form.fasl")
(#x94ca13a80b103ceb970e7d3d65979240 . "./code.idejs/ide-intro.fasl")
(#x89298ba055c7c903ac8d3859aff54149 . "./code.idejs/ide-find-in-files.fasl")
(#x4bc65187d7c45aede8c85397a75283e2 . "./code.idejs/ide-package-list.fasl")
(#xcddb59f58204434e96d17c0eb65f84e4 . "./code.idejs/ide-class-grapher.fasl")
(#x29c4a4cb691acde2a2fe4116aa720ac8 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
